
const { ccclass, property } = cc._decorator;

@ccclass
export default class PieceBase extends cc.Component {
    //叉乘
    private product(a:cc.Vec2,b:cc.Vec2,p:cc.Vec2):number { 
        return (b.x-a.x)*(p.y-a.y)-(b.y-a.y)*(p.x-a.x);
    }
    /**
     * 计算点旋转a度后的坐标 基于原点旋转
     * @param p 初始位置
     * @param a 旋转角度  顺时针
     */
    public rotatePoint(p: cc.Vec2, a: number) :cc.Vec2{
        let theta = a * Math.PI / 180;
        return cc.v2(p.x*Math.cos(theta) + p.y*Math.sin(theta), -p.x*Math.sin(theta) + p.y*Math.cos(theta));
    }
    /**
     * 计算点旋转a度后的坐标 基于某一点旋转
     * @param p 初始点位置 绝对坐标
     * @param o 旋转的锚点坐标
     * @param a 旋转角度  顺时针
     */
    private rotatePointWithPoint(p: cc.Vec2, o: cc.Vec2, a: number): cc.Vec2 {
        let p1 = cc.v2(p.x - o.x, p.y - o.y);//
        let p2 = this.rotatePoint(p1, a);
        //cc.log('p2 x=' + p2.x + ',y=' + p2.y+'  o x='+o.x+',y='+o.y);
        return cc.v2(p2.x + o.x, p2.y + o.y);
    }
    /**
     * 计算点旋转a度后的坐标 基于某一点旋转
     * @param p 初始相对o点位置  相对坐标
     * @param o 旋转的锚点坐标
     * @param a 旋转角度  顺时针
     */
    private rotatePointWithRelativePoint(p: cc.Vec2, o: cc.Vec2, a: number): cc.Vec2 {
        //let p1 = cc.v2(p.x - o.x, p.y - o.y);//
        let p2 = this.rotatePoint(p, a);
        //cc.log('p2 x=' + p2.x + ',y=' + p2.y+'  o x='+o.x+',y='+o.y);
        return cc.v2(p2.x + o.x, p2.y + o.y);
     }
    /**
     * 判断点在三角形区域内
     * a b c逆时针顺序
     * @param a 顶点1
     * @param b 顶点2
     * @param c 顶点3
     * @param p 目标点
    */
   private isInTriangle(a:cc.Vec2,b:cc.Vec2,c:cc.Vec2,p:cc.Vec2): boolean { 
        if(this.product(a,b,p)>=0 && this.product(b,c,p)>=0 && this.product(c,a,p)>=0){
            return true;
        }
        return false;
    }
    /**
     * 判断点在四边形区域内
     * a b c d 逆时针顺序
     * @param a 顶点1
     * @param b 顶点2
     * @param c 顶点3
     * @param d 顶点4
     * @param p 目标点
    */
   private isInQuadrangle(a: cc.Vec2, b: cc.Vec2, c: cc.Vec2, d: cc.Vec2, p: cc.Vec2): boolean{
        if(this.product(a,b,p)>=0 && this.product(b,c,p)>=0 && this.product(c,d,p)>=0 && this.product(d,a,p)>=0){
            return true;
        }
        return false;
    }
    /** 
     * 判读点在扇形区域内
     * @param o 圆心坐标
     * @param r 半径
     * @param a0 扇形起始角度  x正方向为起始角度 逆时针增加 0<a<360
     * @param a 扇形角度  x正方向为起始角度 逆时针增加 0<a<360
     * @param p 目标点
    */
   private isInSector(o:cc.Vec2,r:number,a0:number,a:number,p:cc.Vec2):boolean{
        let radius0 = (a0%360) * Math.PI / 180;
        let radius = a * Math.PI / 180;                 //(0<radius<2*PI)
        let theta = Math.atan2((p.y - o.y), (p.x - o.x));
        theta = theta < 0 ? theta + Math.PI * 2 : theta;//(0<theta<2*PI)
        let state = false;
        if (radius0 < 0 && radius0 + radius <= 0) {
            radius0 += Math.PI * 2;
            state = theta > radius0 && theta < radius + radius0;
        }
        else if (radius0 < 0 && radius0 + radius > 0) {
            state = theta > radius0 + Math.PI * 2 || theta < radius0 + radius;
        }
        else {
            state = theta > radius0 && theta < radius + radius0;
       }
       //cc.log('isInSector state=' + state);
        if (state&&(p.x - o.x) * (p.x - o.x) + (p.y - o.y) * (p.y - o.y) < r * r){
            return true;
        }
        return false;
    }
    /**
     * 判断在圆环区域内
     * @param o  圆心坐标
     * @param r1 内径
     * @param r1 外径
     * @param a0 扇形起始角度  x正方向为起始角度 逆时针增加 0<a<360
     * @param a  扇形角度  x正方向为起始角度 逆时针增加 0<a<360
     * @param p  目标点
     */
    private isInRing(o: cc.Vec2, r1: number, r2: number,a0:number, a: number, p: cc.Vec2): boolean{
        let r0 = Math.sqrt((p.x - o.x) * (p.x - o.x) + (p.y - o.y) * (p.y - o.y));
        let radius = a * Math.PI / 180;
        let radius0 = (a0%360) * Math.PI / 180;
        let theta = Math.atan2((p.y - o.y), (p.x - o.x));
        theta = theta < 0 ? theta + Math.PI * 2 : theta;//(0<theta<2*PI)
        let state = false;
        if (radius0 < 0 && radius0 + radius <= 0) {
            radius0 += Math.PI * 2;
            state = theta > radius0 && theta < radius + radius0;
        }
        else if (radius0 < 0 && radius0 + radius > 0) {
            state = theta > radius0 + Math.PI * 2 || theta < radius0 + radius;
        }
        else {
            state = theta > radius0 && theta < radius + radius0;
        }
        if (r0 >= r1 && r0 <= r2 && state) {
            return true;
        }
        return false;
    }
    /**
     * 带有旋转的三角形区域判断 a b c 逆时针计算
     * @param o 图片坐标
     * @param a 顶点1 配置表数据
     * @param b 顶点2 配置表数据
     * @param c 顶点3 配置表数据
     * @param rotaton 旋转的度数  顺时针
     * @param p 目标点
     */
    isInTriangleWithRotation(o: cc.Vec2, a: cc.Vec2, b: cc.Vec2, c: cc.Vec2, rotaton: number, p: cc.Vec2): boolean { 
        // cc.log('坐标 x=' + o.x + ',y=' + o.y);
        // cc.log('顶点 a x=' + a.x + ',y=' + a.y);
        // cc.log('顶点 b x=' + b.x + ',y=' + b.y);
        // cc.log('顶点 c x=' + c.x + ',y=' + c.y);
        // cc.log('旋转度数 rotation=' + rotaton);
        // cc.log('目标点 p x=' + p.x + ',y=' + p.y);
        let a1 = this.rotatePointWithRelativePoint(a, o, rotaton);
        let b1 = this.rotatePointWithRelativePoint(b, o, rotaton);
        let c1 = this.rotatePointWithRelativePoint(c, o, rotaton);
        //cc.log('顶点 a1 x=' + a1.x + ',y=' + a1.y);
        //cc.log('顶点 b1 x=' + b1.x + ',y=' + b1.y);
        //cc.log('顶点 c1 x=' + c1.x + ',y=' + c1.y);
        if(this.product(a1,b1,p)>=0 && this.product(b1,c1,p)>=0 && this.product(c1,a1,p)>=0){
            return true;
        }
        return false;
    }
    /**
     * 带有旋转的四边形区域判断 a b c d 逆时针计算
     * @param o 图片坐标
     * @param a 顶点1
     * @param b 顶点2
     * @param c 顶点3
     * @param d 顶点4
     * @param rotaton 旋转的度数  顺时针
     * @param p 目标点
     */
    isInQuadrangleWithRotation(o: cc.Vec2, a: cc.Vec2, b: cc.Vec2, c: cc.Vec2, d: cc.Vec2,rotaton:number, p: cc.Vec2):boolean {
        let a1 = this.rotatePointWithRelativePoint(a, o, rotaton);
        let b1 = this.rotatePointWithRelativePoint(b, o, rotaton);
        let c1 = this.rotatePointWithRelativePoint(c, o, rotaton);
        let d1 = this.rotatePointWithRelativePoint(d, o, rotaton);
        if(this.product(a1,b1,p)>=0 && this.product(b1,c1,p)>=0 && this.product(c1,d1,p)>=0 && this.product(d1,a1,p)>=0){
            return true;
        }
        return false;
    }
    /**
     * 带有旋转的扇形区域判断
     * @param o 图片坐标
     * @param r 扇形半径
     * @param a 扇形角度
     * @param rotate 扇形旋转的度数  顺时针
     * @param p 目标点
     */
    isInSectorWithRotation(o:cc.Vec2,r:number,a:number,rotate:number,p:cc.Vec2):boolean { 
        let o0; //扇形圆心坐标
        if (a <= 90) {
            o0 = cc.v2(o.x - r / 2, o.y - r * Math.sin(a * Math.PI / 180) / 2);
        }
        else if (a < 180) {
            o0 = cc.v2(o.x - (r-r * Math.cos(Math.PI-a * Math.PI / 180) ) / 2, o.y - r / 2);
        }
        else if (a <= 270) {
            let edge = r * Math.sin(a * Math.PI / 180 - Math.PI);
            o0 = cc.v2(o.x, o.y - (r - edge) / 2);
        }
        else {
            o0 = o;
        }
        //cc.log("o0 x=" + o0.x + ",y=" + o0.y);
        let o1 = this.rotatePointWithPoint(o0,o, rotate);
        //cc.log("o1 x=" + o1.x + ",y=" + o1.y);
        return this.isInSector(o1, r,-rotate, a,p);
    }
    /**
     * 带有旋转的圆环区域判断
     * @param o 图片坐标
     * @param r 扇形半径
     * @param a 扇形角度
     * @param rotate 扇形旋转的度数  顺时针
     * @param p 目标点
     */
    isInRingWithRotation(o:cc.Vec2,r1: number, r2: number,a:number,rotate:number,p:cc.Vec2):boolean { 
        let o0; //圆环圆心坐标
        let r = r2;
        if (a <= 90) {
            o0 = cc.v2(o.x - r / 2, o.y - r * Math.sin(a * Math.PI / 180) / 2);
        }
        else if (a < 180) {
            o0 = cc.v2(o.x - (r-r * Math.cos(Math.PI-a * Math.PI / 180) ) / 2, o.y - r / 2);
        }
        else if (a <= 270) {
            let edge = r * Math.sin(a * Math.PI / 180 - Math.PI);
            o0 = cc.v2(o.x, o.y - (r - edge) / 2);
        }
        else {
            o0 = o;
        }
        //cc.log("o0 x=" + o0.x + ",y=" + o0.y);
        let o1 = this.rotatePointWithPoint(o0,o, rotate);
        //cc.log("o1 x=" + o1.x + ",y=" + o1.y);
        return this.isInRing(o1,r1, r2,-rotate, a,p);
    }
}
